
#ifndef _mmsystem_h
#define _mmsystem_h

/* Multimedia header file 
    -- not really part of DirectX, but not included in the Win32 API headers.
*/

/* Result code definition */
typedef UINT MMRESULT;

/* Various error values */
#define MAXERRORLENGTH		256	/* Maximum length of error message */
#define MMSYSERR_NOERROR	0
#define TIMERR_NOERROR		MMSYSERR_NOERROR
#define TIMERR_NOCANDO		97

/* Wave out device handle */
typedef HANDLE HWAVEOUT;

/* Have the system choose a wave device */
#define WAVE_MAPPER	((UINT)-1)

/* Specify the type of wave event callback */
#define CALLBACK_FUNCTION	0x00030000
#define CALLBACK_EVENT		0x00050000

/* Messages sent to the waveOut callback function */
#define WOM_DONE	0x3BD

/* The wave buffer header used by waveOut functions */
typedef struct WAVEHDR {
	LPSTR	lpData;
	DWORD	dwBufferLength;
	DWORD	dwBytesRecorded;
	DWORD	dwUser;
	DWORD	dwFlags;
	DWORD	dwLoops;
	struct WAVEHDR *lpNext;
	DWORD	reserved;
} WAVEHDR;

/* WAVEHDR.dwFlags */
#define WHDR_DONE	0x00000001
#define WHDR_BEGINLOOP	0x00000004
#define WHDR_ENDLOOP	0x00000008
#define WHDR_INQUEUE	0x00000010

/* Structure used in querying the capabilities of a wave device */
typedef struct {
	WORD	wMid;
	WORD	wPid;
	UINT	vDriverVersion;
	CHAR	szPname[32];
	DWORD	dwFormats;
	WORD	wReserved1;
	DWORD	dwSupport;
} WAVEOUTCAPS;

/* The only kind of sound data we handle here */
#define WAVE_FORMAT_PCM	1

/* Old wave format structure */
typedef struct {
	WORD    wFormatTag;
	WORD    nChannels;
	DWORD   nSamplesPerSec;
	DWORD   nAvgBytesPerSec;
	WORD    nBlockAlign;
} WAVEFORMAT, *LPWAVEFORMAT;

/* PCM wave format structure */
typedef struct {
	WAVEFORMAT  wf;
	WORD        wBitsPerSample;
} PCMWAVEFORMAT;

/* Wave format structure (used by dsound.h) */
typedef struct {
	WORD  wFormatTag;
	WORD  nChannels;
	DWORD nSamplesPerSec;
	DWORD nAvgBytesPerSec;
	WORD  nBlockAlign;
	WORD  wBitsPerSample;
	WORD  cbSize;
} WAVEFORMATEX, *LPWAVEFORMATEX;

/* Convert a string to a 4 byte multimedia code */
#ifndef mmioFOURCC
#define mmioFOURCC(c0, c1, c2, c3)	\
	((DWORD)(c0)|((DWORD)(c1)<<8)|((DWORD)(c2)<<16)|((DWORD)(c3)<<24))
#endif

/* timer callback function prototype */
typedef void (CALLBACK TIMECALLBACK)(UINT uTimerID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2);

/* What type of timer to set */
#define TIME_ONESHOT    0x0000   /* program timer for single event */
#define TIME_PERIODIC   0x0001   /* program for continuous periodic event */


/* Multimedia timer function declarations */
extern MMRESULT WINAPI timeSetEvent(UINT uDelay, UINT uResolution,
			TIMECALLBACK *fptc, DWORD dwUser, UINT fuEvent);
extern MMRESULT WINAPI timeKillEvent(UINT uTimerID);
extern MMRESULT WINAPI timeBeginPeriod(UINT uPeriod);
extern MMRESULT WINAPI timeEndPeriod(UINT uPeriod);

/* The waveOut* function declarations */
extern MMRESULT WINAPI waveOutOpen(HWAVEOUT *phwo, UINT uDeviceID,
	WAVEFORMATEX *pwfx, DWORD dwCallback, DWORD dwInstance, DWORD fdwOpen);
extern MMRESULT WINAPI waveOutClose(HWAVEOUT hwo);
extern MMRESULT WINAPI waveOutPrepareHeader(HWAVEOUT hwo, WAVEHDR *pwh, UINT cbwh);
extern MMRESULT WINAPI waveOutUnprepareHeader(HWAVEOUT hwo, WAVEHDR *pwh, UINT cbwh);
extern MMRESULT WINAPI waveOutWrite(HWAVEOUT hwo, WAVEHDR *pwh, UINT cbwh);
extern MMRESULT WINAPI waveOutPause(HWAVEOUT hwo);
extern MMRESULT WINAPI waveOutRestart(HWAVEOUT hwo);
extern MMRESULT WINAPI waveOutReset(HWAVEOUT hwo);
extern MMRESULT WINAPI waveOutBreakLoop(HWAVEOUT hwo);
#define waveOutGetDevCaps	waveOutGetDevCapsA
extern MMRESULT WINAPI waveOutGetDevCapsA(UINT uDeviceID, WAVEOUTCAPS *pwoc, UINT cbwoc);
#define waveOutGetErrorText	waveOutGetErrorTextA
extern MMRESULT WINAPI waveOutGetErrorTextA(MMRESULT mmrError, LPSTR pszText, UINT cchText);

#endif /* _mmsystem_h */
