/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997, 1998  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    5635-34 Springhouse Dr.
    Pleasanton, CA 94588 (USA)
    slouken@devolution.com
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_error.c,v 1.4 1998/08/18 07:03:52 slouken Exp $";
#endif

/* Simple error handling in SDL */

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include "SDL_error.h"

#ifdef linux
#define VSNPRINTF	vsnprintf
#endif
#ifdef __WIN32__
#define VSNPRINTF	_vsnprintf
#endif

static char *SDL_error  = NULL;
static int   SDL_errlen = 0;

/* Public functions */

void SDL_SetError (char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
#ifdef VSNPRINTF
	while ( (SDL_error == NULL) ||
				(VSNPRINTF(SDL_error,SDL_errlen,fmt,ap) < 0) ) {
		SDL_errlen += BUFSIZ;
		SDL_error = (char *)realloc(SDL_error, SDL_errlen);
	}
#else
	if ( SDL_error == NULL ) {
		SDL_errlen = BUFSIZ*4;
		SDL_error =  (char *)malloc(SDL_errlen);
		if ( SDL_error == NULL )
			return;
	}
	vsprintf(SDL_error, fmt, ap);
#endif
	va_end(ap);

	/* If we are in debug mode, print out an error message */
#ifdef ERROR_DEBUG
	fprintf(stderr, "SDL_SetError: %s\n", SDL_error);
#else
	if ( getenv("SDL_DEBUG") ) {
		fprintf(stderr, "SDL_SetError: %s\n", SDL_error);
	}
#endif
}

char * SDL_GetError (void)
{
	return(SDL_error ? SDL_error : "");
}

void SDL_ClearError(void)
{
	if ( SDL_error ) {
		SDL_errlen = 0;
		free(SDL_error);
		SDL_error = NULL;
	}
}

/* This is only here because out-of-memory errors can happen everywhere */
void SDL_OutOfMemory(void)
{
	SDL_SetError("Out of memory");
}

#ifdef TEST_ERROR
int main(int argc, char *argv[])
{
	char buffer[BUFSIZ+1];

	SDL_SetError("Hi there!");
	printf("Error 1: %s\n", SDL_GetError());
	SDL_ClearError();
	memset(buffer, '1', BUFSIZ);
	buffer[BUFSIZ] = 0;
	SDL_SetError("This is the error: %s (%f)", buffer, 1.0);
	printf("Error 2: %s\n", SDL_GetError());
	exit(0);
}
#endif
