/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997  Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    5635-34 Springhouse Dr.
    Pleasanton, CA 94588 (USA)
    slouken@devolution.com
*/

#ifdef SAVE_RCSID
static char rcsid =
 "@(#) $Id: SDL_sysaudio.cc,v 1.5 1998/08/27 18:12:41 slouken Exp $";
#endif

/* Allow access to the audio stream on BeOS */

#include <stdio.h>
#include <string.h>
#include <Subscriber.h>
#include <AudioStream.h>

extern "C" {

#include "SDL_audio.h"
#include "SDL_audio_c.h"
#include "SDL_sysaudio.h"

static BDACStream  *audio;
static BSubscriber *ahook;

#define BEOS_AUDIOFORMAT	(AUDIO_S16MSB|AUDIO_STEREO)
#define BEOS_MAXAUDIOVAL	((1<<(16-1))-1)
#define BEOS_MINAUDIOVAL	(-(1<<(16-1)))


/* The BeOS callback for handling the audio buffer */
static bool FillSound(void *unused, char *stream, unsigned long len, void *hdr)
{
	if ( ! SDL_AudioEnabled )
		return(FALSE);

	if ( ! SDL_AudioPaused ) {
		if ( convert.needed ) {
			SDL_LockAudio();
			(*audio_spec.callback)(audio_spec.userdata,
					(Uint8 *)convert.buf, convert.len);
			SDL_UnlockAudio();
			SDL_ConvertAudio(&convert);
			memcpy(stream, convert.buf, convert.len_cvt);
		} else {
			SDL_LockAudio();
			(*audio_spec.callback)(audio_spec.userdata,
					(Uint8 *)convert.buf, len);
			SDL_UnlockAudio();
		}
	}
	return(TRUE);
}

/* Dummy functions -- we don't use thread-based audio */
void SDL_SYS_WaitAudio(void)
{
	return;
}
void SDL_SYS_PlayAudio(void)
{
	return;
}
Uint8 *SDL_SYS_GetAudioBuf(void)
{
	return(NULL);
}

void SDL_SYS_CloseAudio(void)
{
	ahook->ExitStream();
	delete ahook;
	delete audio;
}

int SDL_SYS_OpenAudio(SDL_AudioSpec *spec)
{
	/* If the data is not in native format, we need to convert it */
	spec->format = BEOS_AUDIOFORMAT;
	SDL_CalculateAudioSpec(spec);

	/* Subscribe to the audio stream */
	audio = new BDACStream();
	audio->SetStreamBuffers(spec->size, 2);
	audio->SetSamplingRate((float)spec->freq);
	ahook = new BSubscriber("SDL audio");
	ahook->Subscribe(audio);
	
	/* Enter the audio stream */
	if ( ahook->EnterStream(NULL, TRUE, NULL, FillSound, NULL, TRUE) !=
								B_NO_ERROR ) {
		SDL_SetError("Couldn't enter audio stream");
		return(-1);
	}

	/* We're running! */
	return(1);
}

};	/* Extern C */
