
/* Simple program:  Loop, watching keystrokes
   Note that you need to call SDL_PollEvent() or SDL_WaitEvent() to 
   pump the event loop and catch keystrokes.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "SDL.h"

int main(int argc, char *argv[])
{
	SDL_Event event;
	SDL_keysym keysym;
	int key, nkeys; 
	char *keyname;
	Uint8 *oldkeys, *newkeys;
	int done;

	/* Initialize SDL */
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

	/* Set 640x480 video mode */
	if ( SDL_SetVideoMode(640, 480, 0, SDL_SWSURFACE) == NULL ) {
		fprintf(stderr, "Couldn't set 640x480 video mode: %s\n",
							SDL_GetError());
		exit(2);
	}

	/* Get initial key state */
	newkeys = SDL_GetKeyState(&nkeys);
	oldkeys = (Uint8 *)malloc(nkeys);
	if ( oldkeys == NULL ) {
		fprintf(stderr, "Couldn't allocate memory for key array\n");
		exit(2);
	}
	memcpy(oldkeys, newkeys, nkeys);

	/* Watch keystrokes */
	done = 0;
	while ( !done ) {
		/* Check for events */
		if ( SDL_PollEvent(&event) ) {
			switch (event.type) {
				case SDL_MOUSEBUTTONDOWN:
					/* Any button press quits the app... */
				case SDL_QUIT:
					done = 1;
					break;
				default:
					break;
			}
		}
		/* Check for keystrokes */
		newkeys = SDL_GetKeyState(NULL);
		if ( memcmp(oldkeys, newkeys, nkeys) != 0 ) {
			for ( key = 0; key < nkeys; ++key ) {
				if ( oldkeys[key] ^ newkeys[key] ) {
					keysym.mod = KMOD_NONE;
					keysym.sym = (SDLKey)key;
					SDL_SymToASCII(&keysym, &keyname);
					printf("Key %s:  %s\n", newkeys[key] ?
						"pressed" : "released",keyname);
				}
			}
			memcpy(oldkeys, newkeys, nkeys);
		}
	}
	free(oldkeys);
	exit(0);
}
