
/* Test the thread and mutex locking functions 
   Also exercises the system's signal/thread interaction
*/

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "SDL.h"
#include "SDL_mutex.h"
#include "SDL_thread.h"

#ifdef __WIN32__
#include <windows.h>

#define getpid()	(GetCurrentThreadId()&0xFFFF)
#endif

static SDL_mutex *mutex = NULL;
static pid_t mainthread;
static SDL_Thread *threads[6];

void printid(void)
{
	printf("Process %u:  exiting\n", getpid());
}
	
void terminate(int sig)
{
	printf("Process %u:  raising SIGTERM\n", getpid());
	raise(SIGTERM);
}
void closemutex(int sig)
{
	pid_t id = getpid();
	int i;
	printf("Process %u:  Cleaning up...\n", id == mainthread ? 0 : id);
	for ( i=0; i<6; ++i )
		SDL_KillThread(threads[i]);
	SDL_DestroyMutex(mutex);
	exit(sig);
}
void Run(void *data)
{
	if ( getpid() == mainthread )
		signal(SIGTERM, closemutex);
	while ( 1 ) {
		printf("Process %u ready to work\n", getpid());
		if ( SDL_mutexP(mutex) < 0 ) {
			perror("Couldn't lock mutex");
			exit(1);
		}
		printf("Process %u, working!\n", getpid());
		SDL_Delay(1*1000);
		printf("Process %u, done!\n", getpid());
		if ( SDL_mutexV(mutex) < 0 ) {
			perror("Couldn't unlock mutex");
			exit(1);
		}
	}
}
int main(int argc, char *argv[])
{
	int i;
	int maxproc = 6;

	/* Load the SDL library */
	if ( SDL_Init(0) < 0 ) {
		fprintf(stderr, "%s\n", SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

	if ( (mutex=SDL_CreateMutex()) == NULL ) {
		fprintf(stderr, "Couldn't create mutex\n");
		exit(1);
	}

	mainthread = getpid();
	printf("Main thread: %u\n", mainthread);
	atexit(printid);
	for ( i=0; i<maxproc; ++i ) {
		if ( (threads[i]=SDL_CreateThread(Run, NULL)) == NULL )
			fprintf(stderr, "Couldn't create thread!\n");
	}
	signal(SIGINT, terminate);
	Run(NULL);

	exit(0);	/* Never reached */
}
