/*
    DRAW: An example program for the SDL, maclib, and screenlib libraries
    Copyright (C) 1997  Sam Lantinga

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    5635-34 Springhouse Dr.
    Pleasanton, CA 94588 (USA)
    slouken@devolution.com
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <time.h>

#include "SDL_FrameBuf.h"
#include "Mac_FontServ.h"
#include "Mac_Sound.h"

#include "happy.h"
#include "sprite.h"

#define MAXLOTS	512

/* Globals used by the drawing functions */
FrameBuf *screen = NULL;
FontServ *fontserv = NULL;
Sound *sound = NULL;
MFont *font;
Uint32 white;
Uint32 black;
SDL_Surface *happy_image = NULL;
Sprite *happy = NULL;
Sprite *lots[MAXLOTS];
int LOTS = 0;

void Keyboard(SDL_keysym *keysym)
{
	SDL_Surface *text, *chars;
	char  textbuf[BUFSIZ];
	Uint8 sym;
	char *symname;
	static char charbuf[BUFSIZ];
	static int  pos = 0;

	/* We don't have anything to do if we can't draw */
	if ( ! fontserv )
		return;

	sym = SDL_SymToASCII(keysym, &symname);
	if ( symname ) {
		sprintf(textbuf, "SDL key '%s' pressed!", symname);
	} else {
		sprintf(textbuf, "Unknown SDL key (0x%.2x) pressed!",
							keysym->sym);
	}
	text = fontserv->TextImage(textbuf, font, STYLE_BOLD,
					0xFF, 0xFF, 0xFF, screen->Format());
	if ( text == NULL ) {
		fprintf(stderr, "Couldn't create chars: %s\n",
						fontserv->Error());
		return;
	}

	if ( sym ) {
		if ( sym <= 26 ) {
			charbuf[pos++] = '^';
			charbuf[pos++] = sym-1+'A';
		} else
			charbuf[pos++] = sym;
	}
	charbuf[pos] = 0;
	if (fontserv->TextWidth(charbuf, font, STYLE_NORM) > screen->Width()) {
		charbuf[0] = 0;
		pos = 0;
	}
	chars = fontserv->TextImage(charbuf, font, STYLE_NORM,
					0xFF, 0xFF, 0xFF, screen->Format());
	/* Unprintable text? */
	if ( chars == NULL ) {
		fontserv->FreeText(text);
		return;
	}

	/* Draw text on the background, then update foreground */
	screen->FocusBG();
	screen->FillRect(0, 0,
			screen->Width(), fontserv->TextHeight(font), black);
	screen->FillRect(0, screen->Height()-fontserv->TextHeight(font),
			screen->Width(), fontserv->TextHeight(font), black);
	screen->QueueBlit(0, 0, text, NOCLIP);
	screen->QueueBlit(0, screen->Height()-fontserv->TextHeight(font), chars, NOCLIP);
	screen->Update(1);
	screen->FocusFG();
	fontserv->FreeText(text);
	fontserv->FreeText(chars);
}
void MouseButton(Uint8 state, Uint16 x, Uint16 y)
{
	static Sint16 x1, y1;
	if ( state == SDL_PRESSED ) {
		happy->GetPosition(x1, y1);
		happy->SetVelocity((x-x1)/16, (y-y1)/16);
		x1 = x;
		y1 = y;
	} else {
		/* Draw on the background */
		screen->FocusBG();
		screen->QueueBlit(x1-happy_image->w/2, y1-happy_image->h/2,
								happy_image);
		screen->QueueBlit(x-happy_image->w/2, y-happy_image->h/2,
								happy_image);
		screen->DrawLine(x1, y1, (Sint16)x, (Sint16)y, white);
		screen->Update(1);
		/* Redraw the foreground sprite */
		screen->FocusFG();
		happy->Show();
		screen->Update();
		if ( sound )
			sound->PlaySound(116);		/* Yay! :) */
		printf("."); fflush(stdout);
	}
}
void MouseMotion(Uint8 down, Uint16 x, Uint16 y)
{
	if ( down ) {
		/* Draw a white point on the background */
		screen->FocusBG();
		screen->DrawPoint((Sint16)x, (Sint16)y, white);
		screen->Update(1);		/* Copies to foreground */
		screen->FocusFG();
	}
}
SDL_Surface *ParseXPM(char **xpm)
{
	SDL_Color colors[256];
	int i, w, h, n, bpp;
	Uint8  *pixels;
	Uint8  *mask, maskp;
	SDL_Surface *image;

	sscanf(xpm[0], "%d %d %d %d", &w, &h, &n, &bpp);
	if ( bpp > 1 ) {
		fprintf(stderr, "ParseXPM(): image has too many colors\n");
		return(NULL);
	}
	if ( w%8 ) {
		fprintf(stderr,
		"ParseXPM(): image width must be a multiple of 8 bytes\n");
		return(NULL);
	}

	/* Set the colors */
	memset(colors, 0, sizeof(colors));
	maskp = 0;
	for ( i=1; i<=n; ++i ) {
		char *ptr;
		int   p, j;
		int   skip;
		Uint8 rgb[3];

		p = xpm[i][0];
		if ( i == 1 )
			maskp = p;
		ptr = strrchr(xpm[i], '#');
		ptr += 1;
		skip = strlen(xpm[i])-(ptr-xpm[i]);
		skip /= 3;
		skip -= 1;
		for ( j=0; j<6; ++j ) {
			if ( j%2 )
				rgb[j/2] <<= 4;
			else
				rgb[j/2] = 0;

			if ( (*ptr >= '0') && (*ptr <= '9') ) {
				rgb[j/2] |= *ptr-'0';
			} else
			if ( (*ptr >= 'A') && (*ptr <= 'F') ) {
				rgb[j/2] |= 10+*ptr-'A';
			} else
			if ( (*ptr >= 'a') && (*ptr <= 'f') ) {
				rgb[j/2] |= 10+*ptr-'a';
			}

			if ( j%2 )
				ptr += skip;
			else
				ptr += 1;
		}
		colors[p].r = rgb[0];
		colors[p].g = rgb[1];
		colors[p].b = rgb[2];
//printf("Pixel %d = (0x%2.2X, 0x%2.2X, 0x%2.2X)\n", p, rgb[0], rgb[1], rgb[2]);
	}
	screen->SetPalette(colors);

	/* Set the pixels */
	pixels = new Uint8[w*h];
	for ( i=0; i<h; ++i )
		memcpy(pixels+i*w, xpm[n+i+1], w);

	/* Set the mask */
	mask = new Uint8[(w/8)*h];
	memset(mask, 0, (w/8)*h);
	for ( i=0; i<(w*h); ) {
		if ( pixels[i] != maskp )
			mask[i/8] |= 0x01;
		++i;
		if ( (i%8) != 0 )
			mask[i/8] <<= 1;
	}

	/* Create the image */
	image = screen->LoadImage(w, h, pixels, mask);
	delete[] pixels;
	return(image);
}
void cleanup(int status)
{
	if ( status > 0 ) {
		signal(status, SIG_DFL);
		fprintf(stderr, "Exiting because of fatal signal %d\n", status);
	}
	if ( fontserv ) {
		if ( font )
			delete font;
		delete fontserv;
	}
	if ( sound )
		delete sound;
	if ( happy ) {
		delete happy;
	}
	for ( int i=0; i<LOTS; ++i ) {
		delete lots[i];
	}
	if ( happy_image )
		screen->FreeImage(happy_image);
	if ( screen )
		delete screen;
	exit(status);
}

int main(int argc, char *argv[])
{
	SDL_Event event;
	int   done, i;

	/* Avoid compiler warning */
	argc = argc;

	/* Trap fatal signals for cleanup */
#ifdef SIGSEGV
	signal(SIGSEGV, cleanup);
#endif
#ifdef SIGBUS
	signal(SIGBUS, cleanup);
#endif
	/* Initialize the SDL library (fullscreen video) */
	if ( SDL_Init(SDL_INIT_EVERYTHING) < 0 ) {
		fprintf(stderr, "Couldn't init SDL: %s\n", SDL_GetError());
		cleanup(0);
	}
	atexit(SDL_Quit);

	/* Command line parameter -- the number of random sprites */
	if ( argv[1] ) {
		LOTS = atoi(argv[1]);
		if ( LOTS > MAXLOTS ) {
			LOTS=MAXLOTS;
		}
	}

	/* Load up macintosh resources and initialize. :) */
	fontserv = new FontServ("Maelstrom Fonts");
	if ( fontserv->Error() ) {
		fprintf(stderr, "Warning: %s\n", fontserv->Error());
		delete fontserv;
		fontserv = NULL;
	}
	sound = new Sound("Maelstrom Sounds", 5);
	if ( sound->Error() ) {
		fprintf(stderr, "Warning: %s\n", sound->Error());
		delete sound;
		sound = NULL;
	}
	screen = new FrameBuf();
	if ( screen->Init(640, 480, SDL_SWSURFACE|SDL_FULLSCREEN) < 0 ) {
		fprintf(stderr, "Couldn't initialize video!\n");
		cleanup(0);
	}

	/* Load animations and set colormap before setting callbadks */
	happy_image = ParseXPM(happy_h);
	if ( ! happy_image ) {
		fprintf(stderr, "Couldn't create happy XPM: %s\n",
							screen->Error());
		cleanup(0);
	}
	happy = new Sprite(screen, happy_image,
			screen->Width()/2, screen->Height()/2, 0, 0);
	srand(time(NULL));
	for ( i=0; i<LOTS; ++i ) {
		Sint16 xvel, yvel;

		xvel = (rand()%9)-4;
		yvel = (rand()%9)-4;
		lots[i] = new Sprite(screen, happy_image,
			screen->Width()/2, screen->Height()/2, xvel, yvel);
	}

	/* Set up for text output here -- after ParseXPM() sets colormap */
	white = screen->MapRGB(0xFF, 0xFF, 0xFF);
	black = screen->MapRGB(0x00, 0x00, 0x00);
	if ( fontserv ) {
		font = fontserv->NewFont("Chicago", 12);
		if ( font == NULL ) {
			fprintf(stderr, "Couldn't create font: %s\n",
							fontserv->Error());
			delete fontserv;
			fontserv = NULL;
		}
	}

	/* Set the window clipping for the top and bottom text areas */
	if ( fontserv ) {
		SDL_Rect clipper;

		clipper.x = 0;
		clipper.y = fontserv->TextHeight(font);
		clipper.w = screen->Width();
		clipper.h = screen->Height()-2*fontserv->TextHeight(font);
		screen->ClipBlit(&clipper);
	}

	/* Loop, animating the funny icon */
	done = 0;
	while ( ! done ) {
		/* Handle events */
		while ( screen->PollEvent(&event) ) {
			switch (event.type) {
				case SDL_MOUSEMOTION:
					MouseMotion(event.motion.state,
						event.motion.x, event.motion.y);
					break;
				case SDL_MOUSEBUTTONDOWN:
				case SDL_MOUSEBUTTONUP:
					MouseButton(event.button.state,
						event.button.x, event.button.y);
					break;
				case SDL_KEYDOWN:
					if (event.key.keysym.sym == SDLK_ESCAPE)
						done = 1;
					Keyboard(&event.key.keysym);
					break;
				case SDL_QUIT:
					done = 1;
					break;
			}
		}
		if ( ! LOTS )
			SDL_Delay(10);

		/* Perform animation */
		happy->Hide();
		for ( i=0; i<LOTS; ++i ) {
			lots[i]->Hide();
		}
		for ( i=0; i<LOTS; ++i ) {
			lots[i]->Move();
		}
		for ( i=0; i<LOTS; ++i ) {
			lots[i]->Show();
		}
		happy->Move();
		happy->Show();
		screen->Update();
	}
	/* We received a quit event */
	cleanup(0);

	/* NOTREACHED */
	return(0);
}
